"use strict";

var _transform = require("../transform");

/**
 * @jest-environment jsdom
 */

/*
 * This function is used within the stylisPluginCssCustomProperties plugin.
 * The incoming values are provided by the (internal) stylis CSS compiler.
 * These values have a specific format. Notably, most spaces are removed
 * the ending semicolon (;) is omitted.
 */
describe('getFallbackDeclaration', function () {
  beforeEach(function () {
    // Clear root HTML node styles
    document.documentElement.style = null;
  });
  describe('invalid', function () {
    test('should return undefined if it does not contain var()', function () {
      var result = (0, _transform.getFallbackDeclaration)('font-size:14px');
      expect(result).toEqual(undefined);
    });
    test('should return undefined fallback value is non are provided', function () {
      var result = (0, _transform.getFallbackDeclaration)('font-size: var( --fontSize )');
      expect(result).toEqual(undefined);
    });
  });
  describe('basic', function () {
    test('should use fallback value if provided', function () {
      var result = (0, _transform.getFallbackDeclaration)('font-size: var( --fontSize, 14px )');
      expect(result).toEqual('font-size:14px');
    });
    test('should use fallback value with spaces in-between var()', function () {
      var result = (0, _transform.getFallbackDeclaration)('font-size: var(   --fontSize, 14px          )');
      expect(result).toEqual('font-size:14px');
    });
    test('should use fallback value without spaces in-between var()', function () {
      var result = (0, _transform.getFallbackDeclaration)('font-size: var(--fontSize,14px)');
      expect(result).toEqual('font-size:14px');
    });
    test('should use fallback with parentheses value', function () {
      var result = (0, _transform.getFallbackDeclaration)('filter: var(--blur, blur(10px))');
      expect(result).toEqual('filter:blur(10px)');
    });
    test('should use fallback with nested parentheses value', function () {
      var result = (0, _transform.getFallbackDeclaration)('transform:translate3d(var(--x,5px),var(--y,10px),var( --z, 0))');
      expect(result).toEqual('transform:translate3d(5px,10px,0)');
    });
  });
  describe('nested', function () {
    test('should use nested fallback value if provided', function () {
      var result = (0, _transform.getFallbackDeclaration)('font-size: var( --fontSize, var( --big, 20px ) )');
      expect(result).toEqual('font-size:20px');
    });
    test('should use heavily nested fallback value if provided', function () {
      var result = (0, _transform.getFallbackDeclaration)('font-size: var( --fontSize, var( --one, var( --two, var( --three, var( --four, 20px )))))');
      expect(result).toEqual('font-size:20px');
    });
    test('should use heavily nested fallback with space in-between closing parentheses', function () {
      var result = (0, _transform.getFallbackDeclaration)('font-size: var( --fontSize, var( --one, var( --two, var( --three, var( --four, 20px ))  ) ) )');
      expect(result).toEqual('font-size:20px');
    });
    test('should use heavily nested fallback with parentheses value', function () {
      var result = (0, _transform.getFallbackDeclaration)('filter: var( --fontSize, var( --one, var( --two, var( --three, var( --four, blur(20px) ))  ) ) )');
      expect(result).toEqual('filter:blur(20px)');
    });
  });
  describe(':root fallback', function () {
    test('should not use root fallback if one is provided', function () {
      document.documentElement.style.setProperty('--big', '80px');
      var result = (0, _transform.getFallbackDeclaration)('font-size: var( --fontSize, 20px )');
      expect(result).toEqual('font-size:20px');
    });
    test('should use root fallback if none are provided', function () {
      document.documentElement.style.setProperty('--fontSize', '80px');
      var result = (0, _transform.getFallbackDeclaration)('font-size: var( --fontSize )');
      expect(result).toEqual('font-size:80px');
    });
    test('should use root fallback with nested var()', function () {
      document.documentElement.style.setProperty('--big', '80px');
      var result = (0, _transform.getFallbackDeclaration)('font-size: var( --fontSize, var( --big ) )');
      expect(result).toEqual('font-size:80px');
    });
    test('should use latest root variable', function () {
      document.documentElement.style.setProperty('--big', '80px');
      var dec = 'font-size: var( --fontSize, var( --big ) )';
      var result = (0, _transform.getFallbackDeclaration)(dec); // First pass

      expect(result).toEqual('font-size:80px'); // Update CSS variable on :root

      document.documentElement.style.setProperty('--big', '20px');
      dec = 'font-size: var( --fontSize, var( --big ) )';
      result = (0, _transform.getFallbackDeclaration)(dec); // Second pass

      expect(result).toEqual('font-size:20px');
    });
  });
});